unit uMediaMeta;

interface
uses
  system.generics.collections;


type
  ///  <summary>
  ///    This record is used to store information about the video tracks
  ///    discovered in the target file.
  ///  </summary>
  TVideoTrackInfo = record
    FrameRate: int32;
    xRes: int32;
    yRes: int32;
  end;

  ///  <summary>
  ///    This class provides array-style access to the video track information
  ///    discovered in the target media file.
  ///  </summary>
  TMediaMeta = class
  private
    fVideoTracks: TList<TVideoTrackInfo>;
    fFilename: string;
    fFilepath: string;
  private
    procedure ExtractMeta;
    function getVideoTrackCount: uint32;
    function getVideoTrackInfo(idx: uint32): TVideoTrackInfo;
  public
    constructor Create( Filepath: string; Filename: string ); reintroduce;
    destructor Destroy; override;
  public
    property VideoTrackCount: uint32 read getVideoTrackCount;
    property VideoTrackInfo[ idx: uint32 ]: TVideoTrackInfo read getVideoTrackInfo;
  end;

implementation
uses
  AndroidAPI.JNIBridge,
  AndroidAPI.JNI.JavaTypes,
  AndroidAPI.JNI.Media,
  AndroidAPI.Helpers;

{ TMediaMeta }

constructor TMediaMeta.Create(Filepath, Filename: string);
begin
  inherited Create;
  fFilePath := FilePath;
  fFilename := Filename;
  fVideoTracks := TList<TVideoTrackInfo>.Create;
  ExtractMeta;
end;

destructor TMediaMeta.Destroy;
begin
  fVideoTracks.DisposeOf;
  inherited;
end;

procedure TMediaMeta.ExtractMeta;
var
  f: JFile;
  fis: JFileInputStream;
  fd: JFileDescriptor;
  Extractor: JMediaExtractor;
  Format: JMediaFormat;
  FormatClass: JMediaFormatClass;
  numTracks: int32;
  counter: int32;
  idx: int32;
  mime: JString;
  ARecord: TVideoTrackInfo;
begin
  f := TJFile.JavaClass.init(StringToJString(fFilepath),StringToJString(fFilename));
  fis := TJFileInputStream.JavaClass.init(f);
  fd := fis.getFD;
  Extractor := TJMediaExtractor.JavaClass.init;
  Extractor.setDataSource(fd);
  numTracks := extractor.getTrackCount;
  counter := 0;
  for idx := 0 to pred(numTracks) do begin
    format := extractor.getTrackFormat(idx);
    mime := format.getString( TJMediaFormat.JavaClass.KEY_MIME );
    if mime.startsWith(StringToJString('video/')) then begin
      if format.containsKey(TJMediaFormat.JavaClass.KEY_FRAME_RATE) then begin
        ARecord.FrameRate := format.getInteger(TJMediaFormat.JavaClass.KEY_FRAME_RATE);
        ARecord.xRes := format.getInteger(TJMediaFormat.JavaClass.KEY_WIDTH);
        ARecord.yRes := format.getInteger(TJMediaFormat.JavaClass.KEY_HEIGHT);
        fVideoTracks.Add(ARecord);
      end;
    end;
  end;
end;

function TMediaMeta.getVideoTrackCount: uint32;
begin
  Result := fVideoTracks.Count;
end;

function TMediaMeta.getVideoTrackInfo(idx: uint32): TVideoTrackInfo;
begin
  Result := fVideoTracks.Items[idx];
end;

end.
